import java.util.*;
import java.io.*;

interface myQueue<T extends Comparable <? super T>> {
	void push(T t);
	T top();
	T pop();
}

class BinSearchTree<T extends Comparable<? super T>> {
	T data;
	BinSearchTree<T> left,right,parent;
	int size=1;
	public void add(T t){
		if(t==null) return;
		BinSearchTree<T> cur = this;
		while(true){
			cur.size++;
			if(t.compareTo(cur.data)<0){
				if(cur.left==null){
					cur.left = new BinSearchTree<T>(); cur.left.data = t; cur.left.parent = cur; return;
				}else{
					cur = cur.left;
				}
			}else{
				if(cur.right==null){
					cur.right = new BinSearchTree<T>(); cur.right.data = t; cur.right.parent = cur; return;
				}else{
					cur = cur.right;
				}
			}
		}
	}
	public BinSearchTree<T> findMax(){
		BinSearchTree<T> cur = this;
		while(true){
			if(cur.right == null) return cur;
			cur = cur.right;
		}
	}
	public BinSearchTree<T> findith(int i){ //returns element that is >= i other elements (i.e. element at position i if we had a sorted array)
		BinSearchTree<T> cur = this;
		while(true){
			//System.out.println("i="+i);
			if(i<0 || i>=cur.size){ System.out.println("i="+i); throw new ArrayIndexOutOfBoundsException();}
			int l = (cur.left!=null? cur.left.size:0);
			if(l==i) return cur;
			else if(l>i) cur = cur.left;
			else { cur = cur.right; i = i-(l+1); }
		}
	}
	public String toString(){
		return mytoString(0);
		//return (left!=null? "["+left.toString()+"] " :"") + data.toString() + "(" + size +")" + (right!=null? "["+right.toString()+"]" :"");
	}
	public String mytoString(int p){
		String prefix = "";
		for(int i=0; i<p; i++) prefix +="|-";
		String res = prefix+data.toString() + " ("+ size +")\n";
		if(left!=null) res += left.mytoString(p+1);
		if(right!=null) res += right.mytoString(p+1);
		return res;
	}
}

class ArrayListMedianQueue<T extends Comparable<? super T>> implements myQueue<T> {
	ArrayList<T> mylist;
	public ArrayListMedianQueue(){
		mylist = new ArrayList<>();
	}
	public void push(T t){
		mylist.add(t);
		int i = mylist.size()-1;
		while(i>0){
			if(mylist.get(i-1).compareTo(mylist.get(i))>0){
				T tmp = mylist.get(i);
				mylist.set(i,mylist.get(i-1));
				mylist.set(i-1,tmp);
				i--;
			}else break;
		}
	}
	public T top(){
		return mylist.get(mylist.size()/2);
	}
	public T pop(){
		return mylist.remove(mylist.size()/2);
	}
	public String toString(){
		return mylist.toString();
	}
}

class TreeMedianQueue<T extends Comparable<? super T>> implements myQueue<T> {
	BinSearchTree<T> root;
	public void push(T t){ 
		if(root == null){ root = new BinSearchTree<T>(); root.data = t; return; }
		root.add(t);
	}
	public T top(){ //returns median element
		if(root==null) return null;
		return root.findith(root.size/2).data;
	}
	public T pop(){ 
		if(root == null) return null;
		BinSearchTree<T> res = root.findith(root.size/2);
		T ret = res.data;
		if(root.size==1){ root = null; return ret; }
		if(res.left!=null && res.left.right==null){
			//Easy tree on left side
			res.data = res.left.data; 
			res.left = res.left.left;
			if(res.left!=null) res.left.parent = res;
			for(BinSearchTree<T> tmp = res; tmp!=null; tmp = tmp.parent) tmp.size--;
			return ret;
		}
		if(res.left != null && res.left.right !=null){
			//Non-trivial tree on left side
			BinSearchTree<T> m = res.left.findMax();
			m.parent.right = m.left;
			if(m.left!=null) m.left.parent = m.parent;
			res.data = m.data;
			for(BinSearchTree<T> tmp = m.parent; tmp!=null; tmp = tmp.parent) tmp.size--;
			return ret;
		}
		//Nothing on left side!
		if(res == root){ root = res.right; if(res.right!=null) res.right.parent=null; return ret; }
		if(res.parent.left == res){ res.parent.left = res.right; if(res.right!=null) res.right.parent = res.parent; }
		if(res.parent.right == res){ res.parent.right = res.right; if(res.right!=null) res.right.parent = res.parent;  }
		for(BinSearchTree<T> tmp = res.parent; tmp!=null; tmp = tmp.parent) tmp.size--;
		return ret;
	}
	public String toString() { return root!=null? root.toString(): " [ ]"; }

	public static void main(String [] args){
		//myQueue<Integer> q = new TreeMedianQueue<>();
		myQueue<Integer> q = new ArrayListMedianQueue<>();
		File file = new File(args[0]);
		Scanner sc = null;
		int size=0;
		try{
			sc = new Scanner(file);
			while(sc.hasNextInt()){
				q.push(sc.nextInt()); size++; 
			}
		}catch(IOException e){
			System.out.println("File I/O Error!");
			e.printStackTrace();
		}finally{
			if(sc!=null) sc.close();
		}

		System.out.println("Finished reading file");


		for(int i=0; i<size; i++){
			//System.out.println(q);
			System.out.println("Popped :" + q.pop());
			//System.out.println(q);
			//System.out.println();
		}
	}
}
